@@ -43,6 +43,7 @@ def make_nano():
 
 class MDF(object):
     def __init__(self, *yaml_files, handle=None, model=None, _commit=None,
+                 mdf_schema=None,
                  raiseError=False, logger=logging.getLogger(__name__)):
         """Create a :class:`Model` from MDF YAML files/Write a :class:`Model` to YAML
         :param str|file|url *yaml_files: MDF filenames or file objects, 
@@ -59,6 +60,7 @@ class MDF(object):
             
         self.files = yaml_files
         self.schema = {}
+        self.mdf_schema = mdf_schema
         self._model = model
         self._commit = _commit
         self._terms = {}
@@ -110,7 +112,8 @@ class MDF(object):
             else:
                 vargs.append(f)
                 
-        v = MDFValidator(None, *vargs, raiseError=True)
+        v = MDFValidator(self.mdf_schema, *vargs, raiseError=True)
+        self.mdf_schema = v.load_and_validate_schema()
         self.schema = v.load_and_validate_yaml()
 
     def create_model(self, raiseError=False):
@@ -267,9 +270,14 @@ class MDF(object):
         defns_for = set(ypropdefs.keys())
         for pname in prop_of:
             for ent in prop_of[pname]:
+                force = False
+                # see if a qualified name is defined in propdefs:
                 key = ent.handle+"."+pname
                 ypdef = ypropdefs.get(key)
-                if not ypdef:
+                if ypdef:
+                    # force creation of new prop for a explicitly qualified MDF property
+                    force = True
+                else:
                     key = pname
                     ypdef = ypropdefs.get(pname)
                 if not ypdef:
@@ -280,11 +288,10 @@ class MDF(object):
                         )
                     )
                     continue
-
                 else:
                     if key in defns_for:
                         defns_for.remove(key)
-                prop = self.create_or_merge_prop_from_mdf(ypdef, p_hdl=pname)
+                prop = self.create_or_merge_prop_from_mdf(ypdef, p_hdl=pname, force_create=force)
                 self._model.add_prop(ent, prop)
                 ent.props[prop.handle] = prop
         if defns_for:
@@ -295,11 +302,11 @@ class MDF(object):
             raise RuntimeError("MDF errors found; see log output.")
         return self._model
 
-    def create_or_merge_prop_from_mdf(self, ypdef, p_hdl):
+    def create_or_merge_prop_from_mdf(self, ypdef, p_hdl,force_create):
         init = {"handle": p_hdl,
                 "model": self.handle,
                 "_commit": self._commit}
-        if (init["model"], init["handle"]) in self._props:
+        if not force_create and (init["model"], init["handle"]) in self._props:
             pass  # merge property
         else:
             if 'Desc' in ypdef and ypdef['Desc']:
@@ -316,7 +323,20 @@ class MDF(object):
                     "specify a data type".format(p_hdl=p_hdl)
                 )
                 init["value_domain"] = Property.default("value_domain")
+            # handle union type
+            u_types = None
+            if init["value_domain"] == "union":
+                u_types = init["types"]
+                del init["types"]
+                # reduce to the first value set present in type list
+                specs = [x for x in u_types if x["value_domain"] == "value_set"]
+                if specs:
+                    init.update(specs[0])
+                else:
+                    init["value_domain"] = "union"
             prop = Property(init)
+            if u_types:
+                prop.value_types.extend(u_types)
             if "Tags" in ypdef:
                 for t in ypdef["Tags"]:
                     prop.tags[t] = Tag({"key": t,
@@ -324,6 +344,8 @@ class MDF(object):
                                         "_commit": self._commit})
             if "Term" in ypdef:
                 self.annotate_entity_from_mdf(prop, ypdef["Term"])
+            if force_create:
+                return prop
             self._props[(init["model"], init["handle"])] = prop
         return self._props[(init["model"], init["handle"])]
         
@@ -403,25 +425,38 @@ class MDF(object):
                     )
                 return {"value_domain": json.dumps(typedef)}
         elif isinstance(typedef, list):  # a valueset: create value set and term objs
-            vs = ValueSet({"nanoid": make_nano(), "_commit": self._commit})
-            vs.handle = self.handle + vs.nanoid
-            # interpret boolean values as strings
-            if (isinstance(typedef[0], str) and
-                re.match("^(?:https?|bolt)://", typedef[0])):  # looks like url
-                vs.url = typedef[0]
-            else:  # an enum
+            # could be either a Union or an Enum...
+            if (not isinstance(typedef[0], str) or
+                typedef[0] in self.mdf_schema["defs"]["simpleType"]["enum"]):
+                # guess is a union type
+                ret = []
                 for t in typedef:
-                    if isinstance(t, bool):  # stringify booleans in term context
-                        t = "True" if t else "False"
-                    if t not in self._terms:
-                        self._terms[t] = Term({
-                            "value": t,
-                            "origin_name": self.handle,
-                            "_commit": self._commit
-                        })
-                    vs.terms[self._terms[t].value] = self._terms[t]
-            return {"value_domain": "value_set", "value_set": vs}
+                    ret.append(self.calc_value_domain(t))
+                return {"value_domain": "union", "types":ret}
+            else:
+                vs = ValueSet({"nanoid": make_nano(), "_commit": self._commit})
+                vs.handle = self.handle + vs.nanoid
+                # interpret boolean values as strings
+                if (isinstance(typedef[0], str) and
+                    re.match("^(?:https?|bolt)://", typedef[0])):  # looks like url
+                    vs.url = typedef[0]
+                else:  # an enum
+                    for t in typedef:
+                        if isinstance(t, bool):  # stringify booleans in term context
+                            t = "True" if t else "False"
+                        if t not in self._terms:
+                            self._terms[t] = Term({
+                                "value": t,
+                                "origin_name": self.handle,
+                                "_commit": self._commit
+                            })
+                        vs.terms[self._terms[t].value] = self._terms[t]
+                return {"value_domain": "value_set", "value_set": vs}
         elif isinstance(typedef, str):
+            if typedef not in self.mdf_schema["defs"]["simpleType"]["enum"]:
+                self.logger.warning(
+                    "Type descriptor '{}' not present in MDF schema simpleType definition"
+                    .format(typedef))
             return {"value_domain": typedef}
         else:
             self.logger.warning("Applying default value domain to property '{}'".format(pname))
